<?php include_once("Base-header.php"); ?>

<h1>Description</h1>

<p class="block"><i>libvxmdfmt</i> (and its frontend <i>mdpwlfmt</i>) is used
to beautify MDSYNC transaction logfiles (as generated by
<code>libvxmdsync</code> and frontends like <i>mdsync</i> or <i>xmd</i>) into
nice, user-readable documents like HTML, RTF, Plain Text or others. It takes an
input, an output, and optionally a style template file, as well as a so-called
style name which describes what the beautified file looks like. (Those styles
which do not take a style filename have their output hardcoded in the source
code currently.) Please see the index for detailed descriptions of the
available styles.</p>

<h1>Structures</h1>

<p class="block">Only the fields of interest are listed.</p>

<p class="code"><code>
<b>#</b>include &lt;vitalnix/libvxmdfmt/libvxmdfmt.h&gt;<br />
<br />
<b>struct</b> pwlfmt_workspace {<br />
&nbsp; &nbsp; ...<br />
&nbsp; &nbsp; <b>char *</b>style_name;<br />
&nbsp; &nbsp; <b>char *</b>input_file;<br />
&nbsp; &nbsp; <b>char *</b>output_file;<br />
&nbsp; &nbsp; <b>char *</b>template_file;<br />
&nbsp; &nbsp; <b>void (*</b>report<b>)</b>(<b>const struct</b> pwlfmt_workspace <b>*</b>, <b>int</b>, <b>int</b>);<br />
&nbsp; &nbsp; ...<br />
};
</code></p>

<h1>Function overview</h1>

<p class="code"><code>
<b>#</b>include &lt;vitalnix/libvxmdfmt/libvxmdfmt.h&gt;<br />
<br />
<b>int</b> pwlfmt_new(<b>struct</b> pwlfmt_workspace <b>*</b>ws);<br />
<b>int</b> pwlfmt_process(<b>struct</b> pwlfmt_workspace <b>*</b>ws);<br />
<b>const char *</b>pwlfmt_strerror(<b>int</b> errnum);
</code></p>

<p class="code"><code>
<b>#</b>include &lt;vitalnix/libvxmdfmt/libvxmdfmt.h&gt;<br />
<b>#</b>include &lt;vitalnix/libvxmdfmt/vtable.h&gt;<br />
<br />
<b>struct</b> pwlstyle_vtable <b>*</b>pwlstyles_trav(<b>void **</b>trav);
</code></p>

<h2>pwlfmt_new</h2>

<p class="block">Initializes the workspace and opens the files and style for
I/O. Returns an AEE code.</p>

<table border="1">
  <tr>
    <td class="t1"><code>PWLFMT_ENOSTYLE</code></td>
    <td class="t1">The specified style was not found</td>
  </tr>
  <tr>
    <td class="t2"><code>PWLFMT_EREQTPL</code></td>
    <td class="t2">The style requires a template file, but no file has been
      specified</td>
  </tr>
  <tr>
    <td class="t1"><code>PWLFMT_EEINPUT</code></td>
    <td class="t1">Could not open the input file. The exact error (e.g.
      <code>ENOENT</code>, etc.) is in <code>errno</code>.</td>
  </tr>
  <tr>
    <td class="t2"><code>PWLFMT_EEOUTPUT</code></td>
    <td class="t2">Could not open the output file. The exact error (e.g.
      <code>EPERM</code>, etc.) is in <code>errno</code>.</td>
  </tr>
  <tr>
    <td class="t1"><code>PWLFMT_EINPUTVER</code></td>
    <td class="t1">The input file's format is not supported</td>
  </tr>
</table>

<h2>pwlfmt_process</h2>

<p class="block">Starts the process by reading from the input stream, sorting
the data and printing it out to the output stream. Returns an AEE code.</p>

<h2>pwlfmt_strerror</h2>

<p class="block"><code>pwlfmt_strerror()</code> returns a string describing the
error <code>errnum</code>. If <code>errnum</code> is not an error code as
returned by <code>pwlfmt_new()</code> or <code>pwlfmt_process()</code>, this
function will return <code>NULL</code>.</p>

<h2>pwlstyles_trav</h2>

<p class="block">Allows an application to traverse the list of available
styles. <code>*trav</code> must be NULL on the first call to
<code>pwlstyles_trav()</code>. The function returns a pointer to <code>struct
pwlstyle_vtable</code> for each style, which can then be examined for name,
description and requirement of a style. Returns <code>NULL</code> when there
are no more styles. Example see below.</p>

<h1>Examples</h1>

<p class="code"><code>
<b>struct</b> pwlfmt_workspace w <b>=</b> {<br />
&nbsp; &nbsp; .style_name &nbsp;<b>=</b> "pg_html",<br />
&nbsp; &nbsp; .input_file &nbsp;<b>=</b> "synclog-20060220-083226.log",<br />
&nbsp; &nbsp; .output_file <b>=</b> "synclog-20060220.html",<br />
};<br />
pwlfmt_new(<b>&amp;</b>w);
</code></p>

<p class="code"><code>
<b>struct</b> pwlstyle_vtable <b>*</b>table;<br />
<b>void *</b>trav <b>=</b> NULL;<br />
<b>while</b>((table <b>=</b> pwlstyles_trav(<b>&amp;</b>trav)) <b>!=</b> NULL)<br />
&nbsp; &nbsp; printf("%s: %s%s\n", table<b>-&gt;</b>name, table<b>-&gt;</b>desc,<br />
&nbsp; &nbsp; &nbsp; table<b>-&gt;</b>require_template <b>?</b> " (requires template)" <b>:</b> "");
</code></p>

<?php include_once("Base-footer.php"); ?>
